#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* network monitor.c
 * Monitors the bytes of traffic on a network device
 * 2006-05-14 ed AT s5h DOT net - version 0.1 beta code
 * Use at your own risk. Author not responsible for use.
 *
 * Updates at http://www.s5h.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#define PROCNET "/proc/net/dev"

struct interface_line {
	struct interface_line *next;
	char name[20]; /* over sized */
	long long ibytes;
	long long obytes;
	long long ipackets;
	long long opackets;
	long long ierr;
	long long oerr;
	long long idrop;
	long long odrop;
	long long ififo;
	long long ofifo;
	long long iframe;
	long long oframe;
	long long icompressed;
	long long ocompressed;
	long long imulticast;
	long long omulticast;
};

void *if_name( char *name, struct interface_line *il ) {
	while( il != NULL ) {
		if( strncmp( il->name, name,20 ) == 0 ) {
			return( il );
		}
		il = il->next;
	}
	return( NULL );
}

struct interface_line *find_last( struct interface_line *ptr ) {
	if( ptr == NULL ) {
		return( NULL );
	}

	while( ptr->next != NULL ) {
		ptr = ptr->next;
	}
	return( ptr );
}

void free_list( struct interface_line *i ) {
	struct interface_line *p;
	
	while( i != NULL ) {
		p = i->next;
		free(i);
		i = p;
	}
}


struct interface_line *parse_line( char *line, struct interface_line *lst ) {
	struct interface_line *il;
	struct interface_line *ptr;
	char *colon;
	char *temp = malloc(20);

	colon = strstr( line, ":" );
	if( colon ) {
		*colon = ' ';
	}

	if( sscanf( line, "%s", temp ) == 0 ) {
		fprintf( stderr, "Cannot get interface name\n" );
		exit(EXIT_FAILURE);
	}
	
	il = if_name( temp, lst );
	
	if( il == NULL ) {
		il = malloc( sizeof( struct interface_line ) );
		il->next = NULL;
		
		snprintf( il->name, 20, "%s", temp );
		ptr = find_last( lst );
		
		if( ptr == NULL ) {
			lst = il;
		}
		else {
			ptr->next = il;
		}
	}

	if( sscanf( line, "%s %Ld %Ld %Ld %Ld %Ld %Ld %Ld %Ld %Ld %Ld %Ld %Ld %Ld %Ld %Ld %Ld", 
				il->name, 
				&il->ibytes, &il->ipackets, &il->ierr, 
				&il->idrop, &il->ififo, &il->iframe, 
				&il->icompressed, &il->imulticast, 
				&il->obytes, &il->opackets, &il->oerr, 
				&il->odrop, &il->ofifo, &il->oframe, 
				&il->ocompressed, &il->omulticast 
				) == 0 ) {
		free(il);
		return(NULL);
	}
/*	else {
		printf( "Name %s Bytes in %Ld bytes out %Ld\n", il->name, il->ibytes, il->obytes );
	}*/

	free(temp);
	
	return(lst);
}

struct interface_line *find_il( struct interface_line *ifname, \
		struct interface_line *linked ) {
	struct interface_line *ptr;

	while( ptr != NULL && ptr->next != NULL ) {
		if( strncmp( ptr->name, ifname->name, 20 ) == 0 ) {
			return( ptr );
		}
		ptr = ptr->next;
	}
	return( NULL );
}

void print_list( struct interface_line *p, struct interface_line *pp, \
		struct interface_line *min ) {

	while( p != NULL ) {
		if( pp == NULL ) {
			pp = p;
		}

		printf( 
	
"%6s I:%4LdMiB O:%4LdMiB | I:%4LdKiB O:%4LdKiB | I:%4LdKiB O:%4LdKiB\n",

			p->name, p->ibytes/1024/1024, p->obytes/1024/1024,
			(p->ibytes-pp->ibytes)/1024, 
			(p->obytes-pp->obytes)/1024,
			(p->ibytes-min->ibytes)/1024,
			(p->obytes-min->obytes)/1024 );

		p=p->next;
		pp=pp->next;
		min=min->next;
	}

	printf( "\n\n" );
}

void readdev() {
	FILE *fp;
	char *contents = malloc( 1000 );
	struct interface_line *cur_list = NULL;
	struct interface_line *last_list = NULL;
	struct interface_line *last_min = NULL;
	int count = 0;

	if( ( fp = fopen( PROCNET, "r" ) ) == NULL ) {
		fprintf( stderr, "cannot open %s\n", PROCNET );
		exit(EXIT_FAILURE);
	}

	while( 1 ) {
		if( fseek( fp, 0, SEEK_SET ) ) {
			fprintf( stderr, "error seeking in %s\n", PROCNET );
			exit(EXIT_FAILURE);
		}

		while( fgets( contents, 1000, fp ) ) {
			if( strstr( contents, ":" ) == NULL ) {
				continue;
			}

			cur_list = parse_line( contents, cur_list );
			if( count == 0 ) {
				last_min = parse_line( contents, last_min );
			}
		}

		printf( "   [:: running total ::]   |  [:: reset/sec ::]  |  [:: reset/min ::]\n" );
		print_list( cur_list, last_list, last_min );
		
		free_list( last_list );
		last_list = cur_list;
		cur_list = NULL;
		printf( "  --/ netmon\n" );
		count++;

		if( count == 60 ) {
			free_list( last_min );
			last_min = NULL;
			count=0;
		}

		usleep(1000000);
	}
	fclose(fp);
	
	free(contents);
}

int main() {
	readdev();
	return(EXIT_SUCCESS);
}

